@@ -41,26 +41,11 @@ test_that("reused cached model", {
 
   gen <- odin(code)
   expect_equal(.odin$model_cache_c$list(),
-               hash_string(gen()$ir))
+               hash_string(gen$new()$ir()))
   expect_message(odin(code, verbose = TRUE),
                  "Using cached model")
   expect_silent(odin(code, verbose = FALSE))
 })
 
 
-test_that("reuse shared library", {
-  model_cache_clear()
-  code <- c("deriv(y) <- 0.5",
-            "initial(y) <- 1")
-
-  path <- tempfile()
-  dir.create(path)
-  gen <- odin(code, skip_cache = TRUE, workdir = path)
-  expect_message(
-    odin(code, verbose = TRUE, skip_cache = TRUE, workdir = path),
-    "Using previously compiled shared library")
-  expect_equal(length(dir(path, pattern = paste0(.Platform$dynlib.ext, "$"))),
-               1)
-})
-
 unload_dlls()
